'' +--------------------------------------------------------------------------+
'' | Cluso's Propeller Operating System - RTC object: _RTC  --> _RTC.CMD      |
'' +--------------------------------------------------------------------------+
'' |  Authors:       (c)2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)|
'' |  Modifications: (c)2012 "Cluso99" (Ray Rodrick)                          |
'' |  License:       MIT License - See end of file for terms of use           |
'' +--------------------------------------------------------------------------+
'' This OS module uses an included module to perform most of the housekeeping.
'' Each module is called from, and returns to, the prop binary "_CMD.CMD"
'' Compile with PropTool + "PreSpin 0.3" by Andy Schenk www.insonix.ch/propeller                             
'' See the included code for history and acknowledgments.
''
'' Acknowledgements:
''   Jonathan "lonesock" Dummer for his handle_time routine from the mb_rawb_spi26 object.
''   Michael Green, James Burrows and Jon Williams for I2C routines.
''   James Burrows and Jon Williams for various clock chips including DS1307.
''
'' RR20120326   This module performs an RTC clock function with/without an RTC chip.
''

{-----------------------------------------------------------------------------}
' Date/Time Format:   Year20xx  Month    Date    Hours   Minutes   Seconds   
'                      (00-63)  (1-12)  (1-31)  (00-23)  (00-59)   (00-59) values
'                       000000___0000____00000___00000____000000____000000   
'                       3    2   2  2    2   1   1   1    1    0    0    0 \ bits
'                       1    6   5  2    1   7   6   2    1    6    5    0 /
  DT_LEAPYEAR_MASK   = %000011___0000____00000___00000____000000____000000 
  DT_YEARS_MASK      = %111111___0000____00000___00000____000000____000000
  DT_MONTHS_MASK     = %000000___1111____00000___00000____000000____000000
  DT_DAYS_MASK       = %000000___0000____11111___00000____000000____000000
  DT_HOURS_MASK      = %000000___0000____00000___11111____000000____000000   
  DT_MINUTES_MASK    = %000000___0000____00000___00000____111111____000000   
  DT_SECONDS_MASK    = %000000___0000____00000___00000____000000____111111   

  DT_YEARS_INC       = %000001___0000____00000___00000____000000____000000        
  DT_MONTHS_INC      = %000000___0001____00000___00000____000000____000000 
  DT_MONTHS_MASK2    = %000000___1110____00000___00000____000000____000000 'uses mth=13=%1101
  DT_DAYS_INC        = %000000___0000____00001___00000____000000____000000        
  DT_HOURS_INC       = %000000___0000____00000___00001____000000____000000        
  DT_MINUTES_INC     = %000000___0000____00000___00000____000001____000000   

  DT_YEARS_SHIFT     = 6+6+5+5+4  ' =26
  DT_MONTHS_SHIFT    = 6+6+5+5    ' =22
  DT_DAYS_SHIFT      = 6+6+5      ' =17
  DT_HOURS_SHIFT     = 6+6        ' =12
  DT_MINUTES_SHIFT   = 6          ' =6
{-----------------------------------------------------------------------------}

CON
  _clkmode = _hw#_clkmode_
  _xinfreq = _hw#_xinfreq_

  _nullCharacter                = 0
  _homeCursorCharacter          = 1
  _backspaceCharacter           = 8
  _horizontalTabCharacter       = 9
  _lineFeedCharacter            = 10
  _carriageReturnCharacter      = 13
  _clearScreenCharacter         = 16
  _quoteCharacter               = 34
  _backspaceKeycode             = 200

OBJ
  _hw   : "_hwdef"                      'hardware pin definitions etc
  _hub  : "_hubdef3"                    'fixed hub definitions ($7FFF... down)

  str    : "ASCII0_STREngine.spin"
  std   : "_PCSIO.spin"                 ' pc i/o driver (stays resident, loaded by DOS.BIN)
  stdin : "_STDIN.spin"                 ' use std input driver
  stdout: "_STDOUT.spin"                ' use std output driver

VAR
  long  seconds
  long  date_time
  long  idle_time, last_time, dtime
  byte  buffer[80]
  
  
PUB start | _SIOpins, _SIObaud, cog
  _SIOpins := _hw#_SIOpins
  _SIObaud := _hw#_SIObaud
{------------------------------------------------------------------------------}
'Start serial...
'                 (inRENDEZVOUS, outRENDEZVOUS, serPins,  baud    )
  cog := std.start(_hub#_StdIn,  _hub#_StdOut,  _SIOpins, _SIObaud) ' start driver, returns cog+1
{------------------------------------------------------------------------------}
'StdIO is already running. Just start the stubs.
  stdout.start(_hub#_StdOut)                            ' start the stub
  stdin.start(_hub#_StdIn)                              ' start the stub
{------------------------------------------------------------------------------}

  waitcnt(clkfreq*2 + cnt)
  printstring(string("testing rtc"))
  crlf
  CountSeconds


PUB CountSeconds | Timer, i
    Timer := cnt
    date_time := 12<<dt_years_shift |  3<<dt_months_shift  | 26<<dt_days_shift {
             } | 18<<dt_hours_shift | 44<<dt_minutes_shift | 56
    byte[@buffer] := 0                                  ' terminate buffer just in case
      
    repeat
      waitcnt(Timer += clkfreq/5)          '!!!!!       ' accurate 1 second delay
      UpdateTime

      StringDateTime(date_time, @buffer)
      PrintString(@buffer)
      printstring(string(" tick"))
      crlf
      

PRI printit
      StringDateTime(date_time, @buffer)
      PrintString(@buffer)
      crlf

PRI UpdateTime | flag, days, mths
    date_time++                                                     ' secs +1
    if (date_time & DT_SECONDS_MASK) => 60
      date_time +=  DT_MINUTES_INC                                  ' mins +1
      date_time &= !DT_SECONDS_MASK                                 ' secs =0
      'next: the shift is on the rhs because it evaluates at compile time so it's faster
      if (date_time &  DT_MINUTES_MASK) => (60 << DT_MINUTES_SHIFT) 
        date_time +=  DT_HOURS_INC                                  ' hour +1
        date_time &= !DT_MINUTES_MASK                               ' mins =0
        if (date_time &  DT_HOURS_MASK) => (24 << DT_HOURS_SHIFT)
          date_time +=  DT_DAYS_INC                                 ' day   +1
          date_time &= !DT_HOURS_MASK                               ' hours =0
          days := (date_time & DT_DAYS_MASK  ) >> DT_DAYS_SHIFT     ' extract the days
          mths := (date_time & DT_MONTHS_MASK) >> DT_MONTHS_SHIFT   ' extract the months
          flag~
          if days == 0 '(i.e.=32)           ' 32: it is 0 because we only have 5 bits
            date_time -= DT_MONTHS_INC      ' so remove the rollover to the month
            mths--                          '    "            "
            flag~~                          ' 32: Jan/Mar/May/Jul/Aug/Oct/Dec: mth+1
          elseif days => 31         ' 31: test for 31 days in mth
            if mths => 8            ' if mth Aug/Oct/Dec +1 to make odd; Sep/Nov become even
              mths++                '   Jan/Mar/May/Jly are already odd
            if (mths & 1) == 0      ' now test for an even month which has 30 days
                flag~~              ' even= Apr/Jun/Sep/Nov: mth+1
          elseif days => 29
            if mths == 2                                        ' February?
              if (date_time & DT_LEAPYEAR_MASK) <> 0            ' not leap year?
                flag~~                                          ' not leap year & 29: mth+1
              elseif days => 30
                flag~~                                          ' leap year & 30: mth+1
          if flag                                               ' inc month?
            date_time +=  DT_MONTHS_INC                         ' months +1
            date_time &= !DT_DAYS_MASK                          ' days =0
            date_time +=  DT_DAYS_INC                           ' days +1=1
            if (date_time & DT_MONTHS_MASK) => (13 << DT_MONTHS_SHIFT)
              date_time +=  DT_YEARS_INC                        ' year +1
              date_time &= !DT_MONTHS_MASK2                     ' months =1 (months already odd)
'             date_time &= !DT_MONTHS_MASK                      ' months =0
'             date_time +=  DT_MONTHS_INC                       ' months +1=1
        
PRI ConvertToFAT(dt)
    Return (dt >> 1) + (16 << 25)

PRI ConvertFromFAT(dt)
    Return (dt - (16 << 25)) >> 1    

PRI stringDateTime(dt, stringPointer) 
    byte[stringPointer++] := "2"                             
    byte[stringPointer++] := "0"                              
    Decode((dt & DT_YEARS_MASK) >> DT_YEARS_SHIFT, stringPointer++)    ' set years
    stringPointer++
    byte[stringPointer++] := "/"      
    Decode((dt & DT_MONTHS_MASK) >> DT_MONTHS_SHIFT, stringPointer++)  ' set months
    stringPointer++
    byte[stringPointer++] := "/"
    Decode((dt & DT_DAYS_MASK) >> DT_DAYS_SHIFT, stringPointer++)      ' set days
    stringPointer++
    byte[stringPointer++] := " "
    Decode((dt & DT_HOURS_MASK) >> DT_HOURS_SHIFT, stringPointer++)    ' set hours
    stringPointer++
    byte[stringPointer++] := ":"
    Decode((dt & DT_MINUTES_MASK) >> DT_MINUTES_SHIFT, stringPointer++) 'set minutes
    stringPointer++
    byte[stringPointer++] := ":"
    Decode((dt & DT_SECONDS_MASK), stringPointer++)                    ' set seconds
    byte[++stringPointer] := 0                                         ' string terminator!

PRI Decode(val, stringPointer)
    if val => 0 and val < 100                                          ' semivalidate
      byte[stringPointer++] := (val / 10) + $30                        ' set tens
      byte[stringPointer]   := val - ((val/10)*10) + $30               ' set units
    else
      byte[stringPointer++] := "?"                                     ' ???
      byte[stringPointer]   := "?"                                     ' ???

PUB PrintHexadecimal(number, length)
    PrintString(str.integerToHexadecimal(number, length))

PRI PrintString(pstring)               ' sends string to output devices (vga/serial) without CRLF
    repeat strsize(pstring)
      PrintChar(byte[pstring++])
     
PUB PrintChar(c)                       ' sends character to output devices (vga/serial)
    if c <> _lineFeedCharacter
      stdout.out(c)                                     ' send char  to output device
    elseif byte[_hub#_OS_Rows] & _hub#_LF_MASK       
      stdout.out(c)                                     ' send <lf>  to output device
       
PUB CRLF
    PrintChar(_carriageReturnCharacter)                 ' sends <cr> to output device
    PrintChar(_lineFeedCharacter)                       ' sends <lf> to output device




{{
' This is called more frequently than 1 Hz.
handle_time        
        mov tmp1,cnt            ' get the current timestamp
        add idle_time,tmp1      ' add the current time to my idle time counter
        sub idle_time,last_time ' subtract the last time from my idle counter (hence delta)    
        add dtime,tmp1          ' add to my accumulator, 
        sub dtime,last_time     ' and subtract the old (adding delta)
        mov last_time,tmp1      ' update my "last timestamp"        
        rdlong tmp1,#0          ' what is the clock frequency?
        cmpsub dtime,tmp1 wc    ' if I have more than a second in my accumulator
        addx seconds,#0         ' then add it to "seconds"
}}


{{
' This is called more frequently than 1 Hz.
        tmp1 := cnt                                     ' get the current timestamp
        idle_time += tmp1                               ' add the current time to my idle time counter
        idle_time -= last_time                          ' subtract the last time from my idle counter (hence delta)
        dtime += tmp1                                   ' add to my accumulator,
        dtime -= last_time                              ' and subtract the old (adding delta)
        last_time := tmp1                               ' update my "last timestamp"
        tmp1 := long[$0]                                ' what is the clock frequency?
        if dtime => tmp1                                ' if I have more than a second in my accumulator
          dtime -= tmp1                                 ' subtract 1 seconds worth
          seconds++                                     ' then increment "seconds"
          UpdateDateTime                                ' and update the date/time
          long[_hub#_DateTime] := date_time             'set the hub date & time
'         long[_hub#_Seconds]  := seconds               'set the hub seconds
}}




'' DS1340C driver
''
'' RR20120225   001   start

'' Typically the DS1340C would be...
''   Set by a write   ADDR=%1101000_0 + ADDR=$00 + DATA0 + ... + DATA7
''   Read by  write   ADDR=%1101000_0 + ADDR=$00 +
''     follow read    ADDR=%1101000_1 + DATA0 + ... + DATA7
''   where      DATA0 = seconds
''              DATA1 = minutes
''              DATA2 = hours
''              DATA3 = day
''              DATA4 = date
''              DATA5 = month
''              DATA6 = year
''              DATA7 = control byte
''              DATA8 = trickle charger
''              DATA9 = flag
       

{-----------------------------------------------------------------------------}
{{
'' RR20120226  modify DS1340 object by James Burows for DS1340C

'' ******************************************************************************
'' * DS1340 Object                                                              *
'' * James Burrows May 2006                                                     *
'' * Version 1.2                                                                *
'' ******************************************************************************
''
'' This object provides and example of use of the DS1340 i2c real time clock (RTC)
'' See - for reference:   http://www.maxim-ic.com/quick_view2.cfm/qv_pk/2688
''
'' this object provides the PUBLIC functions:
''  -> Init  - sets up the address and inits sub-objects such
''  -> geti2cError - returns the error passthru from the i2c sub object
''  -> settime - sets the clock time
''  -> setdate - sets the date
''  -> gettime - gets the time into the object - use getday, getmonth etc to read the variables
''  -> getdate - gets the date into the object - see above
''  -> getday/getmonth etc - returns the data got when you call gettime/getdate
''
'' this object provides the PRIVATE functions:
''  -> i2c2bcd - performs integer to BCD conversion
''  -> bcd2int - performs BCD to integer conversion
''
'' this object uses the following sub OBJECTS:
''  -> i2cObject
''
'' Revision History:
''  -> V1 - Release
''      -> V1.1 - Documentation update, slight code tidy-up
''                Changed to include a start status
''                Changed to stop object initializing if device not present on i2cBus
''      -> V1.2 - Updated to allow i2cSCL line driving pass-true to i2cObject
''
'' The default address is %1101_0000

CON

VAR
  long  DS1340_Address
  long  DS1340_Seconds
  long  DS1340_Minutes
  long  DS1340_Hours
  long  DS1340_Days
  long  DS1340_Date
  long  DS1340_Months    
  long  DS1340_Years
  long  started
   
OBJ
  i2cObject     : "i2cObject"

  
PUB Init(_deviceAddress,_i2cSDA,_i2cSCL,_driveSCLLine): okay
  DS1340_Address := _deviceAddress
  i2cObject.init(_i2cSDA, _i2cSCL,_driveSCLLine)

  ' start
  okay := start
  
  return okay

PUB start : okay
  ' start the object
  if started == false
    if i2cObject.devicePresent(DS1340_Address) == true
      started := true
    else
      started := false
  return started     

PUB stop
  ' stop the object
  if started == true
    started := false
    
PUB isStarted : result
  return started

PUB geti2cError : errorCode
  return i2cObject.getError

PUB setTime(ds_hour, ds_minute, ds_seconds)
  if started == true
    ' set the time
    i2cObject.i2cStart
    i2cObject.i2cWrite (DS1340_Address | 0,8)
    i2cObject.i2cwrite(0,8)
    DS1340_Hours   := int2bcd(ds_hour)
    DS1340_Minutes := int2bcd(ds_minute)
    DS1340_Seconds := int2bcd(ds_seconds)  
    i2cObject.i2cWrite (DS1340_Seconds,8)
    i2cObject.i2cWrite (DS1340_Minutes,8)
    i2cObject.i2cWrite (DS1340_Hours,8)        
    i2cObject.i2cStop
  

PUB setDate(ds_day,ds_date,ds_month,ds_year)
  if started == true
  ' set the date 
    i2cObject.i2cStart
    i2cObject.i2cWrite (DS1340_Address | 0,8)
    i2cObject.i2cwrite(3,8)
    DS1340_Days   := int2bcd(ds_day)
    DS1340_Date   := int2bcd(ds_date)
    DS1340_Months := int2bcd(ds_month)
    DS1340_Years  := int2bcd(ds_year)      
    i2cObject.i2cWrite (DS1340_Days,8)  
    i2cObject.i2cWrite (DS1340_Date,8)
    i2cObject.i2cWrite (DS1340_Months,8) 
    i2cObject.i2cWrite (DS1340_Years,8) 
    i2cObject.i2cStop    

  
PUB getDate : ds_seconds | ackbit
  ' get the date bytes from the clock
  if started == true
    i2cObject.i2cStart
    i2cObject.i2cWrite (DS1340_Address | 0,8)
    i2cObject.i2cwrite(3,8)
    i2cObject.i2cStart
    i2cObject.i2cWrite (DS1340_Address | 1,8)
    DS1340_Days   := i2cObject.i2cRead(i2cObject#_i2cACK)
    DS1340_Date   := i2cObject.i2cRead(i2cObject#_i2cACK)
    DS1340_Months := i2cObject.i2cRead(i2cObject#_i2cACK)
    DS1340_Years  := i2cObject.i2cRead(i2cObject#_i2cNAK)
    i2cObject.i2cStop

  
PUB getTime : ds_seconds | ackbit
  ' get the time bytes from the clock
  if started == true
    i2cObject.i2cStart
    i2cObject.i2cWrite (DS1340_Address | 0,8)
    i2cObject.i2cwrite(0,8)
    i2cObject.i2cStart
    i2cObject.i2cWrite (DS1340_Address | 1,8)
    DS1340_Seconds := i2cObject.i2cRead(i2cObject#_i2cACK)
    DS1340_Minutes := i2cObject.i2cRead(i2cObject#_i2cACK)
    DS1340_Hours   := i2cObject.i2cRead(i2cObject#_i2cNAK)
    i2cObject.i2cStop
    return bcd2int(DS1340_Seconds)    
  
  
PUB getHours : result
  return bcd2int(DS1340_Hours)

PUB getMinutes : result
  return bcd2int(DS1340_Minutes)

PUB getSeconds : result
  return bcd2int(DS1340_Seconds)  

PUB getDays : result
  return bcd2int(DS1340_Days)

PUB getDD : result
  return bcd2int(DS1340_Date)

PUB getMonths : result
  return bcd2int(DS1340_Months)

PUB getYears : result
  return bcd2int(DS1340_Years)      
  
pri int2bcd(value) : result
  ' convert integer to BCD (Binary Coded Decimal)
  result := ((value / 10) *16) + (value // 10) 
  return result

pri bcd2int(value) : result
  ' convert BCD (Binary Coded Decimal) to Integer
  result :=((value / 16) *10) + (value // 16) 
  return result
}}

{-----------------------------------------------------------------------------}
{{
con
'the following did not work??
CON
  _clkmode = xtal1 + pll16x 
  _xinfreq = 6_500_000

  siPin    = 31
  soPin    = 30
  sdaPin   = 29
  sclPin   = 28
  baud     = 115200

  addr_r   = %1101000_1         'I2C RTC read addr
  addr_w   = %1101000_0         'I2C RTC write addr


  
VAR
  byte data[16]
  long  rx
  long  z                                    'dummy for a delay

OBJ
  fdx : "FullDuplexSerial_rr004"
  

PUB start | i
  fdx.start(siPin, soPin, 0 , baud)
  waitcnt(5 * clkfreq + cnt)

  fdx.str(string(13, "DS1340C RTC Test v001", 13))

' repeat '>>>>>>>>>

  repeat i from 0 to 5
    rx := fdx.rx
    readRTC
    displayRTC
    waitcnt(5 * clkfreq + cnt)

PUB displayRTC | i, n
  repeat i from 0 to 6
    n := data[i]
    fdx.hex(n,8)
    fdx.tx(" ")
    n := (((data[i] >> 4) & $FF) * 10) + (data[i] & $0F)
    fdx.dec(n)
    fdx.tx(" ")
  fdx.tx(13)

PUB readRTC | i
  writeStart
  writeByte(addr_w)                                     'addr %1101000_0 w
  fdx.hex(z,8)
  fdx.tx(" ")
  writeByte($00)                                        'reg = 0
  repeat i from 0 to 7          
    readByte(i)                                         'read 8 bytes
  writeStop

PRI writeStart
  outa[sclPin]~~                                        'clock ?-->1  \\
  dira[sclPin]~~                                        'clock output ||
  outa[sdaPin]~~                                        'data  ?-->1  || if 0-->1 then stop
  dira[sdaPin]~~                                        'data  output ||
  outa[sclPin]~                                         'clock 1-->0  // start

PRI writeStop
  outa[sdaPin]~                                         'data  ?-->0  \\
  outa[sclPin]~~                                        'clock ?-->1  || clock to high
  outa[sdaPin]~~                                        'data  0-->1  // stop
  dira[sdaPin]~                                         'data  tristate
  dira[sclPin]~                                         'clock tristate
  

PRI readByte(ndx) | i, tmp
  tmp~                             
  repeat i from 0 to 7          
    outa[sclPin]~~                                      'clock 0-->1  \\
    tmp <-=1                                            '             ||
    tmp |= ina[sdaPin]                                  '             || sample bit (msb first)
    outa[sclPin]~                                       'clock 1-->0  //
  outa[sdaPin]~~                                        'data  ?-->1  \\ send ack
  dira[sdaPin]~~                                        'data  output ||
  outa[sclPin]~~                                        'clock 0-->1  ||           
  data[ndx] := tmp                                      'save/delay   || clock ack
  outa[sclPin]~                                         'clock 1-->0  //           
  dira[sdaPin]~                                         'data  tristate
  data[ndx] := tmp

PRI writeByte( cmd ) | i, ack, tmp
  dira[sdaPin]~~                                        'data  ensure output
  tmp := cmd
  cmd ><=8                                              'reverse so msb is b0
  repeat i from 0 to 7    
    outa[sdaPin] := tmp & $01                           'output data value (msb first)
    outa[sclPin]~~                                      'clock 0-->1  \\
    tmp >>= 1                                           'shift data   || clock data
    outa[sclPin]~                                       'clock 1-->0  //
  dira[sdaPin]~                                         'data pin is input (for ack)
  outa[sclPin]~~                                        'clock 0-->1  \\           
  ack := ina[sdaPin]                                    'get ack      || clock ack 
  outa[sclPin]~                                         'clock 1-->0  //           

  z:=ack
  
}}
{-----------------------------------------------------------------------------}

dat                                                   
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}    