'' +--------------------------------------------------------------------------+
'' | Cluso's Propeller Operating System - OS module: _RUN     --> _RUN.CMD    |
'' +--------------------------------------------------------------------------+
'' |  Authors:       (c)2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)|
'' |  Modifications: (c)2012 "Cluso99" (Ray Rodrick)                          |
'' |  License:       MIT License - See end of file for terms of use           |
'' +--------------------------------------------------------------------------+
'' This OS module uses an included module to perform most of the housekeeping.
'' Each module is called from, and returns to, the prop binary "_CMD.CMD"
'' Compile with PropTool + "PreSpin 0.3" by Andy Schenk www.insonix.ch/propeller                             
'' Some OS modules are derived from: 
''      KyeDOS v3 (c) 2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)
'' See the included code for history and acknowledgments.
''
'' RR20120325 This module does......
''                Stops all cogs, boots a ".BIN" file (maintains clkfreq/clkmode)

{==============================================================================}
'' Note: The module __MODULE.spin is not included as we have to load
''         Kye's original unmodified FAT driver to reload the prop!!!
''          (due to a bug I added in v006)
{{
'!!!!!!!!!!!!#include "__MODULE.spin"        ' include the common code for OS modules 

DAT
_ModuleStr    byte      "=== RUN ===",0           'module's name (string)
}}

{==============================================================================}{{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// General Purpose OS Command Module "_MODULE.spin" 
//
// Derived from SD3.01 FATEngine Demo
// Author:        1/2/2011      Kwabena W. Agyeman     
// Modifications: J. Moxham Dr_Acula to create KyeDOS v3.
// Modifications: Cluso99 (Ray Rodrick) break into modules
// Copyright:     (c) 2011 Kwabena W. Agyeman  (Kye)
//                (c) 2011 J. Moxham           (Dr_Acula)
//                (c) 2012 Ray Rodrick         (Cluso99)
// Licence:       MIT (see end of file for details)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Some OS modules are derived from:
//      KyeDOS v3 (c) 2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)
//      Sphinx    (c) 2009-10 by Michael Park and others
// Some concepts and code is from Sphinx (Michael Park) & Spinix (Dave Hein).
// 
// Special thanks to the following authors for their ideas and MIT code:
//   Michael Park, Dave Hein, Kye, James Moxham, Mike Green, Tomas Rokicki,
//   lonesock, hippy, Jeff Ledger, Roger Williams, Michelli Scales, Andy Schenk.
//   My apologies to any others I may have missed.
// And of course to Chip Gracey for the fabulous Prop chip and the Spin Interpreter.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Modifications by Cluso99...
// RR20120318                   Common code included in the OS modules
// RR20120323                   v0.40 change _hubdef
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}

CON
  _clkmode = _hw#_clkmode_
  _xinfreq = _hw#_xinfreq_

  _nullCharacter                = 0
  _homeCursorCharacter          = 1
  _backspaceCharacter           = 8
  _horizontalTabCharacter       = 9
  _lineFeedCharacter            = 10
  _carriageReturnCharacter      = 13
  _clearScreenCharacter         = 16
  _quoteCharacter               = 34
  _backspaceKeycode             = 200

OBJ
  _hw   : "_hwdef"                      ' hardware pin definitions etc
  _hub  : "_hubdef3"                    ' fixed hub definitions ($7FFF... down)

  fat   : "SD-MMC_FATEngine_001.spin"   ' reboot section not changed!!!
  taf   : "SD-MMC_FATEngine_001.spin"
  str   : "ASCII0_STREngine.spin"
' std   : "_PCSIO.spin"                 ' pc i/o driver (stays resident, loaded by DOS.BIN)
  stdin : "_STDIN.spin"                 ' use std input driver
  stdout: "_STDOUT.spin"                ' use std output driver

VAR

   byte CommandLine[80] ' complete command line as typed in
   byte LineOfText1[80] ' general purpose string buffer
   byte LineOfText2[80] ' general purpose string buffer
   byte LineOfText3[80] ' general purpose string buffer

   
PUB start | z,z0,z1,z2,z3,z4
{------------------------------------------------------------------------------}
'StdIO is already running. Just start the stubs.
  stdout.start(_hub#_StdOut)                            ' start the stub
  stdin.start(_hub#_StdIn)                              ' start the stub
{------------------------------------------------------------------------------}
'Start the SD driver (later some of this will also be resident)
  z  :=  long[_hub#_sdPins]
  z0 :=  z        & $1F
  z1 := (z >>  8) & $1F
  z2 := (z >> 16) & $1F
  z3 := (z >> 24) & $1F
'                          (DO, CLK, DI, CS, _SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1)
  ifnot( fat.FATEngineStart(z0, z1,  z2, z3, -1,     -1,     -1,       -1,       -1))
    reboot

  fat.mountPartition(0)                                 ' mount the sd card

  ifnot(fat.partitionMounted)
    PrintString(string("ERROR: Card Not Mounted, stop!"))
    repeat                                              ' LOOP HERE!!!
{------------------------------------------------------------------------------}
'Execute the command
  \str.copy(_hub#_pBuffer_A,@commandline)               ' retrieve commandline 
  \str.tokenizeString(@commandline)
  \executeCommand(@commandline)                         ' execute
{------------------------------------------------------------------------------}
'Return to the command module
  waitcnt(clkfreq / 10 + cnt)                           ' delay
  \fat.bootPartition(string("_CMD.CMD"))
{------------------------------------------------------------------------------}
'Only here on error
  PrintStringCR(string("ERROR: Rebooting..."))
  waitcnt(clkfreq * 2 + cnt)                            ' delay
  reboot
{------------------------------------------------------------------------------}

PUB GetKey
    result := stdin.in                                  ' wait for an input char
    if result== _backspaceKeycode      ' backspace keycode is 200 but convert to 8
      result := _backspaceCharacter 

PUB PrintHexadecimal(number, length)
    PrintString(str.integerToHexadecimal(number, length))

PUB PrintDecimal(number)               ' print the decimal value, useful for debugging
    PrintString(str.integerToDecimal(number, 10))    

PUB PrintStringCR(pstring)             ' sends string to output devices (vga/serial) with CRLF
    PrintString(pstring)                            
    crlf

PUB PrintString(pstring)               ' sends string to output devices (vga/serial) without CRLF
    repeat strsize(pstring)
      PrintChar(byte[pstring++])
     
PUB PrintChar(c)                       ' sends character to output devices (vga/serial)
    if c <> _lineFeedCharacter
      stdout.out(c)                                     ' send char  to output device
    elseif byte[_hub#_OS_Rows] & _hub#_LF_MASK       
      stdout.out(c)                                     ' send <lf>  to output device
       
PUB CRLF
    PrintChar(_carriageReturnCharacter)                 ' sends <cr> to output device
    PrintChar(_lineFeedCharacter)                       ' sends <lf> to output device
{==============================================================================}
     
PRI executeCommand(stringPointer)

  ifnot(str.stringCompareCI(string("run"), stringPointer))      '<-- module's name
    stringPointer := str.tokenizeString(0)              ' get the parameter
    str.copy(stringpointer,@commandline)                
    \runFilename(string(" "), @commandline, string(".BIN")) ' load/run file if found

PRI runFilename(stringptr1, stringptr2, stringptr3)
' concatenate prefix+command+suffix to form a filename
' if file exists, load and run, else return
' uses @lineoftext2
    str.copy(stringptr1, @lineoftext2)
    str.stringConcatenate(@lineoftext2, stringptr2)
    str.stringConcatenate(@lineoftext2, stringptr3)
    str.trimString(@LineOfText2)
    str.stringToUppercase(@LineOfText2)
    if fat.openFile(@LineOfText2, "R")                  ' ensure .BIN file exists
      fat.closeFile
      long[_hub#_HubFree] := _hub#_HUB_RAMSIZE          ' reset to top of hub
      byte[_hub#_OS_Cogs] := 0                          ' reset all cogs
      PrintString(string("Loading program... "))
      PrintString(@LineOfText2)
      crlf
      \fat.bootPartition(@LineOfText2)                  ' load and run
      reboot                                            ' reboot on failure
      
    PrintStringCR(string("File not found."))
    abort 0


dat                                                   
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}